//! Common errors when handling cryptographic functions

/// Crypto errors
#[derive(::thiserror::Error, Debug, Copy, Clone)]
pub enum Error {
    /// Error while parsing key material
    #[error("can't parse key")]
    Parsing,
    /// Not enough data
    #[error("not enough data")]
    NotEnoughData,
    /// Wrong Key type found.
    /// You might have passed rsa keys where x25519 was expected
    #[error("wrong key type")]
    WrongKey,
    /// Unsupported key exchange for this key
    #[error("unsupported key exchange")]
    UnsupportedKeyExchange,
    /// Unsupported cipher
    #[error("unsupported cipher")]
    UnsupportedCipher,
}
